@@ -17,6 +17,23 @@ module B {}
 inline auto x1() { return f<int>(); }
 inline auto z() { return []{}; }
 inline auto x2() { return z(); }
+
+struct Function {
+  template<typename T>
+  Function(T t) : p(new T((T&&)t)) {}
+
+  void *p;
+};
+
+struct Outer {
+  struct Inner {
+    Inner() {}
+    Function f = []{};
+  };
+  Outer(Inner = Inner());
+};
+
+inline void use_nested_1() { Outer o; }
 #pragma clang module end
 #pragma clang module endbuild
 
@@ -30,6 +47,23 @@ inline auto z() { return []{}; }
 inline auto y2() { return z(); }
 inline auto q() { return []{}; }
 inline auto y3() { return q(); }
+
+struct Function {
+  template<typename T>
+  Function(T t) : p(new T((T&&)t)) {}
+
+  void *p;
+};
+
+struct Outer {
+  struct Inner {
+    Inner() {}
+    Function f = []{};
+  };
+  Outer(Inner = Inner());
+};
+
+inline void use_nested_2() { Outer o; }
 #pragma clang module end
 #pragma clang module endbuild
 
@@ -49,3 +83,8 @@ using V = decltype(y3);
 
 #pragma clang module import A
 void (*p)() = f<int>();
+
+void use_nested() {
+  use_nested_1();
+  use_nested_2();
+}
